@@ -50,7 +50,7 @@ def _numeric_urgency(u):
                     'medium'    : 3,
                     'low'       : 4 }
 
-    return urgency_map.get(u.lower(), 0)
+    return urgency_map.get(u.lower(), 99)
 
 
 class ControlStanza:
@@ -168,10 +168,10 @@ class ChangelogParser:
             else:
                 match = self._changelog_header.match(line) if not ancient else None
                 if match:
+                    is_debian_changelog = True
                     if since_version and apt_pkg.version_compare(match.group('version'),
                                                                  since_version) <= 0:
                         break
-                    is_debian_changelog = True
                     urgency = min(_numeric_urgency(match.group('urgency')),
                                   urgency)
                     self._save_entry(line)
@@ -192,7 +192,7 @@ class ChangelogParser:
                     break
 
         if not is_debian_changelog:
-            return None, urgency
+            return None, None
 
         self._save_entry(None)
 
@@ -220,10 +220,12 @@ class Package:
         self.Version = pkgdata.version()
         self.arch = pkgdata.Architecture
 
-    def extract_changes(self, which, since_version=None, reverse=None):
-        '''Extract changelog entries, news or both from the package.
+    def extract_changes(self, which, since_version, reverse):
+        '''Extract changelog and binnmu entries, news or both from the package.
+
+        Unpacks changelog or news files from the binary package, and parses them.
         If since_version is specified, only return entries later than the specified version.
-        returns a sequence of Changes objects.'''
+        Returns a tuple of sequences of Changes objects.'''
 
         news_filenames = self._changelog_variations('NEWS.Debian')
         changelog_filenames = self._changelog_variations('changelog.Debian')
@@ -239,17 +241,60 @@ class Package:
             filenames.extend(changelog_filenames_native)
 
         tempdir = self._extract_contents(filenames)
+        try:
 
-        find_first = lambda acc, fname: acc or self._read_changelog(os.path.join(tempdir, fname), since_version, reverse)
+            find_first = lambda acc, fname: acc or self._read_changelog(os.path.join(tempdir, fname),
+                                                                        since_version, reverse)
 
-        news       = reduce(find_first, news_filenames, None)
-        changelog  = reduce(find_first, changelog_filenames + changelog_filenames_native, None)
-        binnmu     = reduce(find_first, changelog_filenames_binnmu, None)
+            news       = reduce(find_first, news_filenames, None)
+            changelog  = reduce(find_first, changelog_filenames + changelog_filenames_native, None)
+            binnmu     = reduce(find_first, changelog_filenames_binnmu, None)
 
-        shutil.rmtree(tempdir, 1)
+        finally:
+            shutil.rmtree(tempdir, 1)
 
         return (news, changelog, binnmu)
 
+    def extract_changes_via_apt(self, since_version, reverse):
+        '''Run apt-get changelog and parse the downloaded changelog.
+
+        Retrieve changelog using the "apt-get changelog" command, and parse it.
+        If since_version is specified, only return entries later than the specified version.
+        Returns a single sequence of Changes objects or None on downloading or parsing failure.'''
+
+        # Retrieve changelog file and save it in a temporary directory
+        tempdir = tempfile.mkdtemp(prefix='apt-listchanges')
+        changelog_file = os.path.join(tempdir, self.binary + '.changelog')
+        changelog_fd = open(changelog_file, 'w')
+
+        try:
+
+            command = ['apt-get', '-qq', 'changelog', '%s=%s' % (self.binary, self.Version)]
+            ALCLog.debug(_("Calling %(cmd)s to retrieve changelog") % {'cmd': str(command)})
+            subprocess.run(command, stdout=changelog_fd, stderr=subprocess.PIPE, timeout=120, check=True)
+
+        except subprocess.CalledProcessError as ex:
+            ALCLog.error(_('Unable to retrieve changelog for package %(pkg)s; '
+                           + "'apt-get changelog' failed with: %(errmsg)s")
+                           % {'pkg': self.binary,
+                              'errmsg': ex.stderr.decode('utf-8', 'replace') if ex.stderr else str(ex)})
+
+        except Exception as ex:
+            ALCLog.error(_('Unable to retrieve changelog for package %(pkg)s; '
+                          + "could not run 'apt-get changelog': %(errmsg)s")
+                           % {'pkg': self.binary,
+                              'errmsg': str(ex)})
+
+        else:
+            return self._read_changelog(changelog_file, since_version, reverse)
+
+        finally:
+            changelog_fd.close()
+            shutil.rmtree(tempdir, 1)
+
+        return None
+
+
     def _extract_contents(self, filenames):
         tempdir = tempfile.mkdtemp(prefix='apt-listchanges')
 
@@ -281,9 +326,7 @@ class Package:
                     return open(filename, 'rb')
                 break
             except IOError as e:
-                if e.errno == errno.ENOENT:
-                    pass
-                else:
+                if e.errno != errno.ENOENT:
                     raise
         return None
 
@@ -294,7 +337,7 @@ class Package:
 
         entries, urgency = ChangelogParser().parse(fd, since_version)
 
-        if not urgency:
+        if urgency is None:
             return None # not a Debian changelog file
 
         if not entries:
